//+------------------------------------------------------------------+
//|                                        Indicator Pivot Xpert.mq4 |
//|                                       Copyright  2008, Tinytjan |
//|                                                 tinytjan@mail.ru |
//+------------------------------------------------------------------+
#property copyright "Copyright  2008, Tinytjan"
#property link      "tinytjan@mail.ru"

// Magic
#define STUPID 0x60BE57

// Direction defines
#define NONE   0
#define UP     1
#define DN     -1

extern double     Lots              =  0;
extern int        RiskPercentage    =  0;
extern int        Slippage          =  1;
extern int        Fast              =  5;
extern int        Slow              =  13;
extern int        Signal            =  5;
extern int        Price             =  1;
extern int        Sequence          =  10;
extern int        MaxOrders         =  0;

extern int        TrailingPeriod    =  10;
extern int        UseTrailing       =  0;

extern int        Enchancing        =  30;
extern int        UseEnchancing     =  0;

double LotsToBid;
string symbol;

int GetOrdersCount(int MagicNumber, int Type)
{
   int count = 0;
   
   for(int i = 0; i < OrdersTotal(); i++)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != Symbol()) continue;
      // order was opened in another way
      if(OrderMagicNumber() != MagicNumber) continue;
      
      if(OrderType() == Type)
      {
         count++;
      }
   }
   
   return (count);
}

bool NeedToRetry(int errorCode)
{
   switch (errorCode)
   {
      case 1 : return (true);
      case 129 : return (true);
      case 135 : return (true);
      case 138: return (true);
      
      default: return (false);
   }
}

void CloseBuys(int MagicNumber, int Slippage)
{
   for(int i = 0; i < OrdersTotal(); i++)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != Symbol()) continue;
      // order was opened in another way
      if(OrderMagicNumber() != MagicNumber) continue;
      
      if(OrderType() == OP_BUY)
      {
         if(OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, Blue))
         {
            i--;
         }
         else
         {
            if (NeedToRetry(GetLastError()))
            {
               RefreshRates();
               if(OrderClose(OrderTicket(), OrderLots(), Bid, Slippage, Blue))
               {
                  i--;
               }
            }
         }
         RefreshRates();
      }
   }
}

void CloseSells(int MagicNumber, int Slippage)
{
   for(int i = 0; i < OrdersTotal(); i++)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != Symbol()) continue;
      // order was opened in another way
      if(OrderMagicNumber() != MagicNumber) continue;
      
      if(OrderType() == OP_SELL)
      {
         if (OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, Red))
         {
            i--;
         }
         else
         {
            if (NeedToRetry(GetLastError()))
            {
               RefreshRates();
               if(OrderClose(OrderTicket(), OrderLots(), Ask, Slippage, Red))
               {
                  i--;
               }
            }
         }
         RefreshRates();
      }
   }
}

double GetLotsToBid(int RiskPercentage)
{
   double margin = MarketInfo(Symbol(), MODE_MARGINREQUIRED);
   double minLot = MarketInfo(Symbol(), MODE_MINLOT);
   double maxLot = MarketInfo(Symbol(), MODE_MAXLOT);
   double step =   MarketInfo(Symbol(), MODE_LOTSTEP);
   double account = AccountFreeMargin();
   
   double percentage = account*RiskPercentage/100;
   
   double lots = MathRound(percentage/margin/step)*step;
   
   if(lots < minLot)
   {
      lots = minLot;
   }
   
   if(lots > maxLot)
   {
      lots = maxLot;
   }

   return (lots);
}

int GetStopLevel()
{
   return (MathMax(MathRound(MarketInfo(Symbol(), MODE_STOPLEVEL)), 3));
}

void CheckLosses(int MN)
{
   double sellLoss = High[0];
   double buyLoss = Low[0];
   double spread = Ask - Bid;
   
   for (int i = 1; i < TrailingPeriod; i++)
   {
      if (sellLoss < High[i]) sellLoss = High[i];
      if (buyLoss > Low[i]) buyLoss = Low[i];
   }
   
   int stopLevel = GetStopLevel() + Slippage;
   
   if (NormalizeDouble((sellLoss - Bid)/Point, 0) < stopLevel)
   {
      sellLoss = Bid + stopLevel*Point;
   }

   if (NormalizeDouble((Ask - buyLoss)/Point, 0) < stopLevel)
   {
      buyLoss = Ask - stopLevel*Point;
   }

   for(i = 0; i < OrdersTotal(); i++)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != symbol) continue;
      // order was opened in another way
      if(OrderMagicNumber() != MN) continue;
   
      double currentLoss = OrderStopLoss();
      
      if(OrderType() == OP_SELL)
      {
         if (OrderOpenPrice() - sellLoss > spread)
         {
            if (currentLoss > sellLoss || currentLoss == 0)
            {
               OrderModify(OrderTicket(), OrderOpenPrice(), sellLoss, OrderTakeProfit(), 0);
               RefreshRates();
            }
         }
      }

      if(OrderType() == OP_BUY)
      {
         if (OrderOpenPrice() - buyLoss < -spread)
         {
            if (currentLoss < buyLoss || currentLoss == 0)
            {
               OrderModify(OrderTicket(), OrderOpenPrice(), buyLoss, OrderTakeProfit(), 0);
               RefreshRates();
            }
         }
      }
   }
}

double GetLowestPrice(int MagicNumber)
{
      double min = 1000000;
      
   for(int i = 0; i < OrdersTotal(); i++)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != Symbol()) continue;
      // order was opened in another way
      if(OrderMagicNumber() != MagicNumber) continue;
      
      if(OrderType() == OP_SELL)
      {
         double current = OrderOpenPrice();
         if (current < min)
         {
            min = current;
         }
      }
   }
   return (min);
}

double GetHighestPrice(int MagicNumber)
{
      double max = 0;
      
   for(int i = 0; i < OrdersTotal(); i++)
   {
      // already closed
      if(OrderSelect(i, SELECT_BY_POS) == false) continue;
      // not current symbol
      if(OrderSymbol() != Symbol()) continue;
      // order was opened in another way
      if(OrderMagicNumber() != MagicNumber) continue;
      
      if(OrderType() == OP_BUY)
      {
         double current = OrderOpenPrice();
         if (current > max)
         {
            max = current;
         }
      }
   }
   return (max);
}

void OpenBuy(int MN, int Target, int Loss)
{
   if (UseEnchancing == 1) if (Ask < GetHighestPrice(MN) + Enchancing*Point) return;
   
   double TP = 0;
   if (Target > 0)
   {
      TP = Ask + Target*Point;
   }

   double SL = 0;
   if (Loss > 0)
   {
      SL = Ask - Loss*Point;
   }
   
   if (Lots == 0) LotsToBid = GetLotsToBid(RiskPercentage);
   
   OrderSend(Symbol(), OP_BUY, LotsToBid, Ask, Slippage, SL, TP, NULL, MN, 0, Blue);
}

void OpenSell(int MN, int Target, int Loss)
{
   if (UseEnchancing == 1) if (Bid > GetLowestPrice(MN) - Enchancing*Point) return;

   double TP = 0;
   if (Target > 0)
   {
      TP = Bid - Target*Point;
   }

   double SL = 0;
   if (Loss > 0)
   {
      SL = Bid + Loss*Point;
   }
   
   if (Lots == 0) LotsToBid = GetLotsToBid(RiskPercentage);
   
   OrderSend(Symbol(), OP_SELL, LotsToBid, Bid, Slippage, SL, TP, NULL, MN, 0, Red);
}

void Check()
{
   double signal = iCustom(symbol, 0, "MyMACD_V2", Fast, Slow, Signal, Price, Sequence, 0, 1);
   double closeSignal = iCustom(symbol, 0, "MyMACD_V2", Fast, Slow, Signal, Price, Sequence, 2, 1);

   if (signal > 0)
   {
      CloseSells(STUPID, Slippage);
      if ((GetOrdersCount(STUPID, OP_BUY) < MaxOrders || MaxOrders == 0) )
      {
         OpenBuy(STUPID, 0, 0);
      }
   }
   
   if (signal < 0)
   {
      CloseBuys(STUPID, Slippage);
      if ((GetOrdersCount(STUPID, OP_SELL) < MaxOrders || MaxOrders == 0))
      {
         OpenSell(STUPID, 0, 0);
      }
   }
}

int init()
{
   LotsToBid = Lots;
   symbol = Symbol();
}

int start()
{
   if (Fast >= Slow) return(0);
   //if (ShortBars > LongBars) return(0);
   // Trading only when tick is opening
   if (Volume[0] == 1)
   {
      // Check for open new orders and close current ones
      if (UseTrailing == 1) CheckLosses(STUPID);
      Check();
   }

   return(0);
}